
'''
   Copyright © 2019  Atos Spain SA. All rights reserved.
  
   This file is part of the ReMAP platform.
  
   The ReMAP platform is free software: you can redistribute it 
   and/or modify it under the terms of GNU GPL v3.
   
   THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT ANY WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT, 
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN ACTION OF CONTRACT, TORT 
   OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
   OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  
   See README file for the full disclaimer information and LICENSE file 
   for full license information in the project root.  
  '''
from flask import Flask, json, request 
import os
import csv
from shutil import copyfile

print("ReMAP SDK Testing Environment")

DATA_DIR='/app/'

if not os.path.exists(DATA_DIR):
    os.makedirs(DATA_DIR)

copyfile("./data/dataset.csv", "/app/dataset.csv")
copyfile("./data/tasks.csv", "/app/tasks.csv")
copyfile("./data/schedule.csv", "/app/schedule.csv")
copyfile("./data/metadata.json", "/app/metadata.json")
copyfile("./data/sdk_config.json", "/app/sdk_config.json")
copyfile("./data/secret", "/app/secret")

print("configuration loaded")
#-------------------------------------

api = Flask(__name__)

@api.route('/mock/getToken', methods=['POST'])
def getToken():
  req_data = {"access_token":"eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJSZU1BUCBNb2NrIEF1dGggU2VydmVyIiwiaWF0IjoxNTk3MTQ0MjkzLCJleHAiOjE2Mjg2ODAyOTMsImF1ZCI6IlRlc3RpbmdfU0RLIiwic3ViIjoidGVzdGluZ19zZGtAcmVtYXAuZXUiLCJHaXZlbk5hbWUiOiJBbGV4IiwiU3VybmFtZSI6IkdhcmNpYSIsIkVtYWlsIjoiYWxlamFuZHJvLmdhcmNpYUBhdG9zLm5ldCIsIlJvbGUiOlsiU29mdHdhcmUgZW5naW5lZXIiLCJQcm9qZWN0IEFkbWluaXN0cmF0b3IiLCJSZXNlYXJjaGVyIl19.sJBKiT1gOkaXKliiWXFF2jKH_aqGf_AHS27wZwkcsoM"}
  return req_data

@api.route('/mock/output', methods=['POST'])
def post_output():
  print("SDK Output received")

  req_data = request.get_json()
  print(req_data)
  return json.dumps(req_data)

@api.route('/mock/scheduleOutput', methods=['POST'])
def post_schedule_output():
  print("SDK Output received")
  return json.dumps('ok')

if __name__ == '__main__':
    print("Mock ReMAP server for SDK Testing")
    api.run()


#!/usr/bin/python


'''
   Copyright © 2019  Atos Spain SA. All rights reserved.
  
   This file is part of the ReMAP platform.
  
   The ReMAP platform is free software: you can redistribute it 
   and/or modify it under the terms of GNU GPL v3.
   
   THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT ANY WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT, 
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN ACTION OF CONTRACT, TORT 
   OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
   OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  
   See README file for the full disclaimer information and LICENSE file 
   for full license information in the project root.  
  '''

import json

import pandas as pd
import numpy as np
import json
# Import ReMAP SDK module
from remapSDK import remapSDK


class model:

    def __init__(self, system, param):
        self.parameters = param
        self.system = system
        self.sensor_data = pd.DataFrame()
        self.RUL = np.NaN
        self.std = np.NaN

        # Simple linear regression model:
        self.slope = 13.0

        # Standard deviation of RUL is proportional to prognostic horizon
        self.std_slope = 1.0

        # Define how we express the RUL
        self.unit = "CYCLES"

    def calculate_rul(self, sensor_data):
        self.sensor_data = sensor_data

        # Just take the last value from the relevant parameter and calculate RUL
        sensor_value = sensor_data[self.parameters[0]].values[-1]
        self.RUL = sensor_value * self.slope
        self.std = sensor_value * self.std_slope
        return [self.RUL, self.std]


 ### 1. Initialize ###
print('#'*10 + ' INITIALIZING ' + '#'*10)
remapSdk = remapSDK.Sdk()

print('#'*10 + ' WP5 PoC ' + '#'*10)

# Live data
print('#'*10 + ' READING LIVE DATA ' + '#'*10)
# Sensor data
data_table = pd.DataFrame()
data_table = data_table.append(pd.read_csv(
    remapSdk.getDataset()), ignore_index=True)
data_table['timestamp'] = pd.to_datetime(
    data_table['timestamp'])


# Prognostics
print('#'*10 + ' RUNNING PROGNOSTICS ' + '#'*10)

# Select last flight of this aircraft
last_departure = data_table.sort_values(by='timestamp').iloc[-1].timestamp
data_last_flight = data_table[data_table.timestamp == last_departure]

# Access models from all aircraft. Feed data to these models. Calculate RUL
start_time = remapSdk.getStartTime()
print("start_time: "+start_time)

end_time = remapSdk.getEndTime()
print("end_time: "+end_time)

tailno = remapSdk.getTailNumber()
print("tailno: "+tailno)


# model(system, params)
metadata = pd.DataFrame()
metadata = metadata.append(remapSdk.getMetadata())
parameters = metadata[metadata.type == "number"].to_dict('records')
parametersInference = [str(parameters[0]['parameter']['name'])]
print()
print("Parameters for inference:")
print(parametersInference)
print("PartNo("+parametersInference[0]+"): " +
      remapSdk.getParamPartNumber(parametersInference[0]))
new_model = model('brake_1', parametersInference)

[RUL, std] = new_model.calculate_rul(data_last_flight)

jsonoutput = {
    "RULUnit": new_model.unit,
    "RULValue": RUL,
    "confidenceInterval": {
        "max": RUL + std,
        "min": RUL - std
    },
    "distributionType": "Normal",
    "distributionParams": [
        {
            "name": "mean",
            "value": RUL
        },
        {
            "name": "std",
            "value": std
        }
    ],
    "failureMode": 'FAILURE'
}

output = remapSdk.sendOutput(jsonoutput)
print(json.dumps(json.loads(output), indent=2))
print('#'*10 + ' End WP5 PoC ' + '#'*10)

print('')

print('#'*10 + ' WP6 PoC ' + '#'*10)
aircraft = remapSdk.getAircraft()
print("aircraft: "+json.dumps(aircraft, indent=2))

fleet = remapSdk.getFleet()
print("fleet: "+json.dumps(fleet, indent=2))

slots = remapSdk.getSlots()
print("slots file: "+slots)
slots_table = pd.DataFrame()
slots_table = slots_table.append(pd.read_csv(
    slots), ignore_index=True)
print(slots_table)

workforce = remapSdk.getWorkforce()
print("workforce file: "+workforce)
workforce_table = pd.DataFrame()
workforce_table = workforce_table.append(pd.read_csv(
    workforce), ignore_index=True)
print(workforce_table)

matrix = remapSdk.getMatrix()
print("matrix file: "+matrix)
matrix_table = pd.DataFrame()
matrix_table = matrix_table.append(pd.read_csv(
    matrix), ignore_index=True)
print(matrix_table)

tasks = remapSdk.getTasks()
print("tasks file: "+tasks)
tasks_table = pd.DataFrame()
tasks_table = tasks_table.append(pd.read_csv(
    tasks), ignore_index=True)
print(tasks_table)

workpackages = remapSdk.getWorkpackages()
print("workpackages file: "+workpackages)
workpackages_table = pd.DataFrame()
workpackages_table = workpackages_table.append(pd.read_csv(
    workpackages), ignore_index=True)
print(workpackages_table)

aircraft_type = remapSdk.getAircraftType()
print("aircraft_type file: "+aircraft_type)
aircraft_type_table = pd.DataFrame()
aircraft_type_table = aircraft_type_table.append(pd.read_csv(
    aircraft_type), ignore_index=True)
print(aircraft_type_table)

skills = remapSdk.getSkills()
print("skills file: "+skills)
skills_table = pd.DataFrame()
skills_table = skills_table.append(pd.read_csv(
    skills), ignore_index=True)
print(skills_table)

output = remapSdk.sendScheduleOutput(remapSdk.getTasks())
output = remapSdk.sendScheduleOutput(remapSdk.getSlots())

print('#'*10 + ' End WP6 PoC ' + '#'*10)



'''
   Copyright © 2019  Atos Spain SA. All rights reserved.
  
   This file is part of the ReMAP platform.
  
   The ReMAP platform is free software: you can redistribute it 
   and/or modify it under the terms of GNU GPL v3.
   
   THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT ANY WARRANTY OF ANY KIND, 
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT, 
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN ACTION OF CONTRACT, TORT 
   OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
   OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  
   See README file for the full disclaimer information and LICENSE file 
   for full license information in the project root.  
  '''

import csv
import json
from datetime import datetime
import requests


class Sdk:
    '''ReMAP SDK for RUL Algorithms'''

    DIR_BASE = '/app/'
    DATASET_FILE = 'dataset.csv'
    SLOTS_FILE = 'slots.csv'
    WORKFORCE_FILE = 'workforce.csv'
    MATRIX_FILE = 'matrix.csv'
    TASKS_FILE = 'tasks.csv'
    WORKPACKAGES_FILE = 'workpackages.csv'
    AIRCRAFT_TYPE_FILE = 'aircraft_type.csv'
    SKILLS_FILE = 'skills.csv'
    METADATA_FILE = 'metadata.json'
    CONFIG_FILE = 'sdk_config.json'
    METADATA = 'metadata'

    config = None
    start_date = None
    end_date = None
    tailNumber = None
    metadata = None
    aircraft = None
    fleet = None

    def __init__(self):
        '''SDK constructor'''
        with open(self.DIR_BASE+self.CONFIG_FILE) as File:
            self.config = json.load(File)

    def getStartTime(self):
        '''get the start time of metadata'''
        if self.start_date is None:
            with open(self.DIR_BASE+self.METADATA_FILE) as file:
                self.metadata = json.load(file)
            self.start_date = self.metadata['startTime']
        else:
            pass
        return self.start_date

    def getEndTime(self):
        '''get the end time of metadata'''
        if self.end_date is None:
            with open(self.DIR_BASE+self.METADATA_FILE) as file:
                self.metadata = json.load(file)
            self.end_date = self.metadata['endTime']
        else:
            pass
        return self.end_date

    def getTailNumber(self):
        if self.tailNumber is None:
            if self.metadata is None:
                with open(self.DIR_BASE+self.METADATA_FILE) as file:
                    self.metadata = json.load(file)
            self.tailNumber = self.metadata['tailNumber']
        else:
            pass
        return self.tailNumber

    def getAircraft(self):
        ''' get the aircraft '''
        if self.aircraft is None:
            if self.metadata is None:
                with open(self.DIR_BASE+self.METADATA_FILE) as file:
                    self.metadata = json.load(file)
            self.aircraft = self.metadata['aircraft']
        else:
            pass
        return self.aircraft

    def getFleet(self):
        ''' get the fleet '''
        if self.fleet is None:
            if self.metadata is None:
                with open(self.DIR_BASE+self.METADATA_FILE) as file:
                    self.metadata = json.load(file)
            self.fleet = self.metadata['fleet']
        else:
            pass
        return self.fleet

    def getMetadata(self):
        with open(self.DIR_BASE+self.METADATA_FILE) as file:
            self.metadata = json.load(file)
        return self.metadata['metadata']

    def getReplacements(self):
        if self.metadata is None:
            with open(self.DIR_BASE+self.METADATA_FILE) as file:
                self.metadata = json.load(file)
        return self.metadata['replacements']

    def getDataset(self):
        return self.DIR_BASE+self.DATASET_FILE

    def getSlots(self):
        return self.DIR_BASE+self.SLOTS_FILE

    def getWorkforce(self):
        return self.DIR_BASE+self.WORKFORCE_FILE

    def getMatrix(self):
        return self.DIR_BASE+self.MATRIX_FILE

    def getTasks(self):
        return self.DIR_BASE+self.TASKS_FILE

    def getWorkpackages(self):
        return self.DIR_BASE+self.WORKPACKAGES_FILE

    def getAircraftType(self):
        return self.DIR_BASE+self.AIRCRAFT_TYPE_FILE

    def getSkills(self):
        return self.DIR_BASE+self.SKILLS_FILE

    # return the component PartNo of the parameter passed as param
    def getParamPartNumber(self, param):
        PartNo = "P/N Not Found"
        if self.metadata is None:
            with open(self.DIR_BASE+self.METADATA_FILE) as file:
                self.metadata = json.load(file)
        self.metadata = self.metadata[self.METADATA]
        for x in self.metadata:
            if x.__contains__('parameter'):
                parameter = x['parameter']
                if parameter is not None:
                    name = parameter['name']
                    if name == param:
                        component = parameter['component']
                        PartNo = component['partNo']
        return PartNo

    def sendOutput(self, jsonoutput):
        secret = self.__getClientSecret()
        payload = jsonoutput
        payload['serial'] = self.config['serial']
        payload['dataset'] = self.config['dataset']
        payload['model'] = self.config['model']
        payload['status'] = "FINISHED"
        payload['runnerId'] = self.config['runnerId']

        token = self.__getKCToken(secret)
        json_dump = json.dumps(payload)
        headers = {"Content-Type": "application/json",
                   "Authorization": "Bearer "+token}
        r = requests.post(self.config['OUTPUT_URL'],
                          headers=headers, data=json_dump)
        return (r.text)

    def sendHealthIndicator(self, jsonoutput):
        secret = self.__getClientSecret()
        payload = jsonoutput
        payload['serial'] = self.config['serial']
        payload['dataset'] = self.config['dataset']
        payload['model'] = self.config['model']
        payload['status'] = "FINISHED"
        payload['runnerId'] = self.config['runnerId']

        token = self.__getKCToken(secret)
        json_dump = json.dumps(payload)
        headers = {"Content-Type": "application/json",
                   "Authorization": "Bearer "+token}
        r = requests.post(self.config['HEALTH_OUTPUT_URL'],
                          headers=headers, data=json_dump)
        return (r.text)

    def sendScheduleOutput(self, schedule_file):
        secret = self.__getClientSecret()
        token = self.__getKCToken(secret)
        headers = {"Content-Type": "application/json",
                   "Authorization": "Bearer "+token}

        with open(schedule_file, 'rb') as f:
            r = requests.post(self.config['SCHEDULE_OUTPUT_URL'],
                              headers=headers, files={'output.xls': f})
        return (r.text)

    def __getKCToken(self, secret):

        url = self.config['KC_TOKEN_URL']
        payload = {
            "client_id": self.config['CLIENT_ID'],
            "grant_type": "client_credentials",
            "client_secret": secret}
        r = requests.post(url, data=payload)
        res = json.loads(r.text)
        return (res['access_token'])

    def __getClientSecret(self):

        secret = "Secret Not Found"
        with open(self.DIR_BASE+'secret') as file:
            secret = file.read()
        return secret
